// Player.java

import ch.aplu.tcp.*;
import ch.aplu.jgamegrid.*;

public class Player extends TcpAgent
  implements GGMouseListener
{
  private static final String VERSION = "1.1";
  private static final String nickname = "TicTacToePlayer";
  private volatile boolean isMyTurn = false;
  private volatile boolean isOver = false;
  private char mySign;
  private char teammateSign;
  private Guide guide = null;
  private final Board board = new Board();
  private final Location hideLoc = new Location(-1, -1);

  public Player(String sessionID, String bridgeName)
  {
    super(sessionID, bridgeName);
    board.addMouseListener(this, GGMouse.lPress
      | GGMouse.move | GGMouse.leave | GGMouse.enter);
    board.setTitle(nickname + " V" + VERSION);
    addTcpAgentListener(new TcpAgentAdapter()
    {
      public void dataReceived(String source, int[] data)
      {
        switch (data[0])
        {
          case Command.IN_PROGRESS:
            board.setStatusText("Gameroom occupied.\nTerminating now...");
            TcpTools.delay(4000);
            System.exit(0);
            break;
          case Command.TEAMMATE_QUIT:
            board.setStatusText("Teammate left game room.\nWaiting for another teammate...");
            board.clear();
            isMyTurn = false;
            isOver = false;
            break;
          case Command.TEAMMATE_WAIT:
            board.setStatusText("Entered in game room.\nWaiting for teammate...");
            break;
          case Command.MARK_X:
            mySign = 'X';
            teammateSign = 'O';
            guide = new Guide('X');
            break;
          case Command.MARK_O:
            mySign = 'O';
            teammateSign = 'X';
            guide = new Guide('O');
            break;
          case Command.START_FIRST:
            board.clear();
            board.addActor(guide, hideLoc);
            board.setStatusText("Game started.\nClick to put your mark.");
            isOver = false;
            isMyTurn = true;
            break;
          case Command.START_SECOND:
            board.clear();
            board.addActor(guide, hideLoc);
            board.setStatusText("Game started.\nWait for partner's move.");
            isOver = false;
            isMyTurn = false;
            break;
          case Command.MOVE:
            board.setStatusText("It's your move.\nClick to put your mark.");
            isMyTurn = true;
            Location loc = new Location(data[1], data[2]);
            board.addActor(new Mark(teammateSign), loc);
            break;
          case Command.WON:
            isOver = true;
            board.setStatusText("Game over. You won.\nClick in board to restart.");
            break;
          case Command.LOST:
            isOver = true;
            board.setStatusText("Game over. You lost.\nClick in board to restart.");
            break;
          case Command.TIE:
            isOver = true;
            board.setStatusText("Game over. It is a draw.\nClick in board to restart.");
            break;
          case Command.RESTART:
            board.setStatusText("Teammate wants to restart.\nClick in board to accept.");
            break;
        }
        if (isOver)
        {
          isMyTurn = false;
          board.addActor(new Actor("sprites/gameover.gif"), new Location(1, 1));
        }
      }

      public void notifyBridgeConnection(boolean connected)
      {
        if (!connected)
        {
          board.setStatusText("Game server stopped.\nTerminating now...");
          TcpTools.delay(4000);
          System.exit(0);
        }
      }
    });
    connectToRelay(nickname);
    if (!isBridgeConnected())
    {
      board.setStatusText("Connected, but game server not found.\nTerminating now...");
      TcpTools.delay(4000);
      System.exit(0);
      return;
    }
  }

  public boolean mouseEvent(GGMouse mouse)
  {
    if (isOver && mouse.getEvent() == GGMouse.lPress)
    {
      isOver = false;
      board.clear();
      board.setStatusText("Restart requested.\nWait for partner's approval.");
      send("", Command.RESTART);
      return true;
    }

    if (isMyTurn)
    {
      Location loc = board.toLocation(mouse.getX(), mouse.getY());
      switch (mouse.getEvent())
      {
        case GGMouse.move:
        case GGMouse.enter:
          if (!guide.getLocation().equals(loc))
          {
            if  (board.getOneActorAt(loc, Mark.class) == null)
              guide.setLocation(loc);
            else
              guide.setLocation(hideLoc);
            board.refresh();
          }
          break;
        case GGMouse.leave:
          guide.setLocation(hideLoc);
          board.refresh();
          break;
        case GGMouse.lPress:
          if (board.getOneActorAt(loc, Mark.class) == null)
          {
            isMyTurn = false;
            board.setStatusText("Game in progress.\nWait for partner's move.");
            board.addActor(new Mark(mySign), loc);
            send("", Command.MOVE, loc.x, loc.y);
          }
          break;
      }
    }
    return true;
  }
}
